@@ -193,7 +193,14 @@ class SQLMapping(object):
                 else:
                     m_Value = m_Value + str(m_row_struct[m_nRowPos])
 
-        m_ResultSQL = re.sub(p_Key, m_Value, p_szSQL, flags=re.DOTALL)
+        try:
+            m_ResultSQL = re.sub(p_Key, m_Value, p_szSQL, flags=re.DOTALL)
+        except re.error as ex:
+            print("[WARNING] Invalid regex pattern in ReplaceSQL. " +
+                  "[" + str(p_Key) + "]:[" + m_Value + "]:[" +
+                  p_szSQL + "]  " +
+                  repr(ex), file=self.Console)
+            raise ex
         return m_ResultSQL
 
     def RewriteSQL(self, p_szTestScriptFileName, p_szSQL):
@@ -217,10 +224,14 @@ class SQLMapping(object):
                 try:
                     if re.match(m_Mapping_Contents[0], m_TestScriptFileName):       # 文件名匹配
                         for (m_Key, m_Value) in m_Mapping_Contents[1]:              # 内容遍历
-                            m_New_SQL = self.ReplaceSQL(m_New_SQL, m_Key, m_Value)
+                            try:
+                                m_New_SQL = self.ReplaceSQL(m_New_SQL, m_Key, m_Value)
+                            except re.error as ex:
+                                print("[WARNING] Invalid regex pattern in ReplaceSQL. ",
+                                      file=self.Console)
                 except re.error as ex:
-                    print("[WARNING] Invalid regex pattern. " +
-                          "[" + str(m_Mapping_Contents[0]) + " : " + m_TestScriptFileName + " : " +
+                    print("[WARNING] Invalid regex pattern in filename match. " +
+                          "[" + str(m_Mapping_Contents[0]) + "]:[" + m_TestScriptFileName + "]:[" +
                           m_MappingFiles + "]  " +
                           repr(ex), file=self.Console)
                     print("[WARNING] Your mapping config has been ignored.")
@@ -264,10 +275,22 @@ def SQLAnalyze(p_SQLCommandPlainText):
     # 首先备份原始的SQL
     SQLCommandsWithComments = copy.copy(SQLCommands)
 
-    # 从SQLCommands中删除所有的注释信息
-    m_bInCommentBlock = False
+    # 从SQLCommands中删除所有的注释信息，但是不能删除ECHO中的注释信息
+    m_bInCommentBlock = False         # 是否在多行注释内
+    m_bInEchoSQL = False              # 是否在ECHO语句内部
     for m_nPos in range(0, len(SQLCommands)):
         while True:  # 不排除一行内多个注释的问题
+            # 首先处理特殊的ECHO信息
+            if m_bInEchoSQL:
+                # ECHO信息已经结束
+                if re.match(r'echo\s+off', SQLCommands[m_nPos], re.IGNORECASE):
+                    m_bInEchoSQL = False
+                break
+            # 如果当前是ECHO文件开头，进入ECHO模式
+            if re.match(r'echo\s+.*', SQLCommands[m_nPos], re.IGNORECASE):
+                m_bInEchoSQL = True
+                break
+
             # 如果存在行内的块注释, 且当前不是在注释中
             if not m_bInCommentBlock:
                 # 如果存在行注释
@@ -331,16 +354,48 @@ def SQLAnalyze(p_SQLCommandPlainText):
     m_bInBlockSQL = False
     # 是否在多行语句内部
     m_bInMultiLineSQL = False
+    # 是否在ECHO语句内部
+    m_bInEchoSQL = False
 
     # 拼接好的SQL
     m_NewSQL = None
     # 拼接好的SQL，包含了注释信息
     m_NewSQLWithComments = None
+    # 当前的Echo信息
+    m_EchoMessages = None
 
     # 下一段注释将从m_NewSQLWithCommentPos的m_NewSQLWithCommentsLastPos字符开始
     m_NewSQLWithCommentsLastPos = 0            # 注释已经截止到的行中列位置
     m_NewSQLWithCommentPos = 0                 # 注释已经截止到的行号
+
     for m_nPos in range(0, len(SQLCommands)):
+        # 首先处理特殊的ECHO信息
+        if m_bInEchoSQL:
+            # ECHO信息已经结束
+            if re.match(r'echo\s+off', SQLCommands[m_nPos], re.IGNORECASE):
+                # 添加ECHO信息到解析后的SQL中
+                SQLSplitResults.append(m_EchoMessages)
+                SQLSplitResultsWithComments.append(m_EchoMessages)
+                SQLSplitResults.append(SQLCommands[m_nPos])
+                SQLSplitResultsWithComments.append(SQLCommands[m_nPos])
+                m_EchoMessages = None
+                m_bInEchoSQL = False
+                continue
+            # 当前是一段ECHO信息
+            if m_EchoMessages is None:
+                m_EchoMessages = SQLCommands[m_nPos]
+                continue
+            else:
+                m_EchoMessages = m_EchoMessages + "\n" + SQLCommands[m_nPos]
+                continue
+        # 如果当前是ECHO文件开头，则当前的ECHO语句作为一个SQL返回，随后进入ECHO模式
+        if re.match(r'echo\s+.*', SQLCommands[m_nPos], re.IGNORECASE):
+            SQLSplitResults.append(SQLCommands[m_nPos])
+            SQLSplitResultsWithComments.append(SQLCommands[m_nPos])
+            m_bInEchoSQL = True
+            continue
+
+        # 正常处理其他SQL
         if not m_bInMultiLineSQL:  # 没有在多行语句中
             # 这是一个单行语句, 要求单行结束， 属于内部执行，需要去掉其中的注释信息
             if re.match(r'^(\s+)?loaddriver\s|^(\s+)?connect\s|^(\s+)?start\s|^(\s+)?set\s|'
@@ -630,7 +685,7 @@ def SQLAnalyze(p_SQLCommandPlainText):
         # 去掉行尾的空格
         SQLSplitResults[m_nPos] = SQLSplitResults[m_nPos].rstrip()
         # 去掉行尾的最后一个分号, “但是如果是END；结尾的，最后的；不能去掉”
-        if SQLSplitResults[m_nPos][-1:] == ';' and not SQLSplitResults[m_nPos].endswith("END;"):
+        if SQLSplitResults[m_nPos][-1:] == ';' and not SQLSplitResults[m_nPos].upper().endswith("END;"):
             SQLSplitResults[m_nPos] = SQLSplitResults[m_nPos][:-1]
 
     # 去掉注释信息中的最后一个回车换行符
@@ -648,36 +703,30 @@ def SQLAnalyze(p_SQLCommandPlainText):
         if len(SQLSplitResults[m_nPos]) == 0:
             # 这里为一个注释信息，解析注释信息中是否包含必要的tag
             for line in SQLSplitResultsWithComments[m_nPos].splitlines():
-                # [Hint]  SQLElapsedTime less than 5
-                # [Hint]  order
-                # [Hint]  Feature:XXXX
-                # [Hint]  SQLID:XXXX
-                # [Hint]  SQLGROUP:XXXX
-                matchObj = re.search("^(\s+)?--(\s+)?\[Hint\](\s+)?SQLElapsedTime\s+less\s+than\s+(\d+)", line,
-                                     re.IGNORECASE|re.DOTALL)
-                if matchObj:
-                    m_TimeLimit = int(matchObj.group(4))
-                    m_SQLHint["TimeLimit"] = m_TimeLimit
-
-                matchObj = re.search("^(\s+)?--(\s+)?\[Hint\](\s+)?order", line,
-                                     re.IGNORECASE|re.DOTALL)
+                # [Hint]  order           -- SQLCli将会把随后的SQL语句进行排序输出，原程序的输出顺序被忽略
+                # [Hint]  Feature:XXXX    -- 相关SQL的特性编号，仅仅作为日志信息供查看
+                # [Hint]  SQLID:XXXX      -- 相关SQL的SQL编号ID，仅仅作为日志信息供查看
+                # [Hint]  SQLGROUP:XXXX   -- 相关SQL的SQL组编号，仅仅作为日志信息供查看
+                matchObj = re.search(r"^(\s+)?--(\s+)?\[Hint\](\s+)?order", line,
+                                     re.IGNORECASE | re.DOTALL)
                 if matchObj:
                     m_SQLHint["Order"] = True
 
-                matchObj = re.search("^(\s+)?--(\s+)?\[Hint\](\s+)?Feature:(.*)", line,
-                                     re.IGNORECASE|re.DOTALL)
+                matchObj = re.search(r"^(\s+)?--(\s+)?\[Hint\](\s+)?Feature:(.*)", line,
+                                     re.IGNORECASE | re.DOTALL)
                 if matchObj:
                     m_SQLHint["Feature"] = matchObj.group(4)
 
-                matchObj = re.search("^(\s+)?--(\s+)?\[Hint\](\s+)?SQLID:(.*)", line,
-                                     re.IGNORECASE|re.DOTALL)
+                matchObj = re.search(r"^(\s+)?--(\s+)?\[Hint\](\s+)?SQLID:(.*)", line,
+                                     re.IGNORECASE | re.DOTALL)
                 if matchObj:
                     m_SQLHint["SQLID"] = matchObj.group(4)
 
-                matchObj = re.search("^(\s+)?--(\s+)?\[Hint\](\s+)?SQLGROUP:(.*)", line,
-                                     re.IGNORECASE|re.DOTALL)
+                matchObj = re.search(r"^(\s+)?--(\s+)?\[Hint\](\s+)?SQLGROUP:(.*)", line,
+                                     re.IGNORECASE | re.DOTALL)
                 if matchObj:
                     m_SQLHint["SQLGROUP"] = matchObj.group(4)
+
             m_SQLHints.append({})
         else:
             # 这里是一个可执行SQL信息
@@ -685,5 +734,4 @@ def SQLAnalyze(p_SQLCommandPlainText):
             m_SQLHints.append(m_SQLHint)
             m_SQLHint = {}
 
-
     return not m_bInMultiLineSQL, SQLSplitResults, SQLSplitResultsWithComments, m_SQLHints
